unit ChartDlg;

interface

uses
  ServerData, dOPCDA,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMXTee.Engine, FMXTee.Canvas,
  FMXTee.Chart.Functions, FMXTee.Series, FMXTee.Procs, FMXTee.Chart, FMX.Menus,
  FMX.Controls.Presentation, FMX.MultiView;

type
  TChartForm = class(TCoreForm)
    Chart: TChart;
    Series1: TFastLineSeries;
    PopupMenu1: TPopupMenu;
    MenuItem1: TMenuItem;
    Label1: TLabel;
    InfoLabel: TLabel;
    DefLabel: TLabel;
    MenuClear: TMenuItem;
    MenuGrid: TMenuItem;
    procedure MenuItem1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MenuClearClick(Sender: TObject);
    procedure MenuGridClick(Sender: TObject);
  private
    fItem  : TdOPCItem;
    fServer: TdOPCDAClient;
    function GetItem: TdOPCItem;
    procedure SetColor;
  public
    procedure ShowChartValues(OPCServer  : TOPCDAServer; Item: TdOPCItem);
    property  OPCItem : TdOPCItem read GetItem;
    procedure AddValue(Item: TdOPCItem);
    procedure Clear;
  end;

var
  ChartForm: TChartForm;

implementation

{$R *.fmx}

{ TChartForm }

procedure TChartForm.AddValue(Item: TdOPCItem);
begin
  if Item = fItem then
  try
    // save only the last 400 values if not visible
    if not Chart.Visible and (Series1.Count > 400) then
      Series1.Clear;

    Series1.Add(Item.Value);
    InfoLabel.Text := IntToStr(Series1.Count)+'/ '+TimeToStr(Time);
  except
    InfoLabel.Text := 'convert error!';
    fItem := nil;
    Series1.Clear;
  end;
end;

procedure TChartForm.SetColor;
var
  DefColor: TAlphaColor;
//  O : TFmxObject;
begin
  DefColor := TAlphaColorRec.White;
//  O := DefLabel.FindStyleResource('text');
//  if O <> nil then
//    ShowMessage('Hello');
  //DefColor := DefLabel.S
  Chart.LeftAxis.LabelsFont.Color := DefColor;
  Chart.LeftAxis.Axis.Color := DefColor;
  Chart.BottomAxis.LabelsFont.Color := DefColor;
  Chart.BottomAxis.Axis.Color := DefColor;
end;

procedure TChartForm.Clear;
begin
  Series1.Clear;
  FItem := nil;
end;

procedure TChartForm.FormCreate(Sender: TObject);
begin
  inherited;
  AddPopup('Item Chart',PopupMenu1);
  Series1.Clear;
  SetColor;
  Chart.LeftAxis.Grid.Visible   :=  false;
  Chart.BottomAxis.Grid.Visible :=  false;
end;

function TChartForm.GetItem: TdOPCItem;
begin
  result := fItem;
end;

procedure TChartForm.MenuClearClick(Sender: TObject);
begin
  Series1.Clear;
end;

procedure TChartForm.MenuGridClick(Sender: TObject);
begin
  Chart.LeftAxis.Grid.Visible   :=  not Chart.LeftAxis.Grid.Visible;
  Chart.BottomAxis.Grid.Visible :=  not Chart.BottomAxis.Grid.Visible;
end;

procedure TChartForm.MenuItem1Click(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TChartForm.ShowChartValues(OPCServer: TOPCDAServer; Item: TdOPCItem);
begin
  fServer := OPCServer;
  if fItem  <> Item then
    Series1.Clear;
  fItem   := Item;
  if assigned(fItem) then
    HeaderLabel.Text := fItem.ItemID
  else
    HeaderLabel.Text := 'unknown';
end;

end.
