unit GroupEditDlg;

interface

uses
  //KeyboardFormManager,
  Serverdata, dOPCDA, System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMX.Edit, FMX.Menus, System.Generics.Collections,
  FMX.Controls.Presentation, FMX.MultiView;

type
  TGroupEditForm = class(TCoreForm)
    PopupMenu1: TPopupMenu;
    MenuOk: TMenuItem;
    MenuCancel: TMenuItem;
    Label1: TLabel;
    eGroupName: TEdit;
    Label2: TLabel;
    eUpdateRate: TEdit;
    Label3: TLabel;
    eDeadBand: TEdit;
    Label4: TLabel;
    eWaitTime: TEdit;
    Label5: TLabel;
    eHoldTime: TEdit;
    VertScrollBox1: TVertScrollBox;
    Layout1: TLayout;

    procedure FormCreate(Sender: TObject);
    procedure MenuCancelClick(Sender: TObject);
    procedure MenuOkClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    fOldEditIndex : integer;
    procedure GroupToForm(Group: TdOPCGroup);
    procedure FormToGroup(Group: TdOPCGroup);
    procedure FormDefault;
    function GetNewName(GName: string): string;
  public
    //FormMoveManager: TKeyboardFormManager;
    fMode : integer;
    fServerItem: TServerItem;
    procedure AddGroup(ServerItem: TServerItem);
    procedure EditGroup(ServerItem: TServerItem; Groupname: string);
  end;

var
  GroupEditForm: TGroupEditForm;

implementation

{$R *.fmx}

procedure TGroupEditForm.AddGroup(ServerItem: TServerItem);
begin
  fMode := 0;
//  eGroupName.Visible := true;
  eGroupName.Enabled := true;
//  ActiveControl := eGroupName;
  fServerItem := ServerItem;
  //FormEmpty;
  FormDefault;
 // Layout1.Align := TAlignLayout.alNone;
end;

function GetField(Edit: TEdit; Default: string) : string; overload;
begin
  if Edit.Text = '' then
    result := Default
  else
    result := Edit.Text;
end;

function GetField(Edit: TEdit; Default: integer): integer;  overload;
begin
  if Edit.Text = '' then
    result := Default
  else
    result := StrToInt(Edit.Text);
end;

function GetField(Edit: TEdit; Default: single): single;  overload;
begin
  if Edit.Text = '' then
    result := Default
  else
    result := StrToFloat(Edit.Text);
end;


//procedure TGroupEditForm.FormEmpty;
//begin
//  eGroupName.Text  := '';
//  eUpdateRate.Text := '';
//  eDeadBand.Text   := '';
//  eWaitTime.Text   := '';
//  eHoldTime.Text   := '';
//end;

function TGroupEditForm.GetNewName(GName: string): string;
var
  counter : integer;
  Ok      : boolean;
begin
  Counter := 0;
  repeat
    Inc(Counter);
    Ok := fServerItem.Server.OPCGroups.GetOPCGroup(GName+IntToStr(counter)) = nil;
  until Ok;
  result := GName+IntToStr(counter);
end;

procedure TGroupEditForm.FormDefault;
begin
  eGroupName.Text  := GetNewName('MyGroup');
  eUpdateRate.Text := '1000';
  eDeadBand.Text   := format('%.2f',[0.00]);
  eWaitTime.Text   := '0';
  eHoldTime.Text   := '0';
end;


procedure TGroupEditForm.FormDestroy(Sender: TObject);
begin
  inherited;
  //FormMoveManager.DisposeOf;
end;

procedure TGroupEditForm.GroupToForm(Group: TdOPCGroup);
begin
  eGroupName.Text  := Group.Name;
  eUpdateRate.Text := IntToStr(Group.UpdateRate);
  eDeadBand.Text   := format('%.2f',[Group.DeadBand]);
  eWaitTime.Text   := IntToStr(Group.XMLWaittime);
  eHoldTime.Text   := IntToStr(Group.XMLHoldtime);
end;

procedure TGroupEditForm.FormToGroup(Group: TdOPCGroup);
begin
  Group.Name        :=  GetField(eGroupName,'MyGroup') ;
  Group.UpdateRate  :=  GetField(eUpdateRate,1000);
  Group.DeadBand    :=  GetField(eDeadBand,0.00);
  Group.XMLWaittime :=  GetField(eWaitTime,0)  ;
  Group.XMLHoldtime :=  GetField(eHoldTime,0);
end;

procedure TGroupEditForm.EditGroup(ServerItem: TServerItem; Groupname: string);
var
  Group : TdOPCGroup;
begin
   fMode := 1;
   eGroupName.Enabled := false;
//   ActiveControl := eUpdateRate;
   fServerItem := ServerItem;
   Group := ServerItem.Server.OPCGroups.GetOPCGroup(GroupName);
   if Group <> nil then
    GroupToForm(Group)
   else
   begin
     ShowMessage('group not found');
     close;
   end;

end;

procedure TGroupEditForm.FormCreate(Sender: TObject);
begin
  inherited;
  fMode := 0;
  fOldEditIndex := -1;
  AddPopup('Group && Items',PopupMenu1);
  //FormMoveManager := TKeyboardFormManager.Create(self,VertScrollBox1,Layout1);
end;

procedure TGroupEditForm.MenuCancelClick(Sender: TObject);
begin
  inherited;
  BackToParentForm;
end;


procedure TGroupEditForm.MenuOkClick(Sender: TObject);
var
  Group: tdOPCGroup;
begin
  inherited;
  if trim(eGroupName.text) = '' then
  begin
    ShowMessage('please enter group name');
    exit;
  end;
  if fMode = 1  then
    Group := fServerItem.Server.OPCGroups.GetOPCGroup(eGroupName.Text)
  else
    Group := fServerItem.Server.OPCGroups.Add(eGroupName.Text);
  if Group = nil then
    raise Exception.Create('group name error');
  FormToGroup(Group);
  BackToParentForm(true);
end;

end.
