unit GroupsDlg;
interface

uses
  dOPCDA, WriteDlg, ChartDlg, BrowseDlg,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMX.ListView.Types, FMX.Header,
  FMX.ListView, FMX.Menus, FMX.TabControl, System.Rtti, FMX.Grid, ServerData,
  FMX.Controls.Presentation, FMX.MultiView, System.Threading, FMX.Grid.Style,
  FMX.ScrollBox;

type
  TGroupsForm = class(TCoreForm)
    PopupMenu1: TPopupMenu;
    MenuAddGroup: TMenuItem;
    MenuDelGroup: TMenuItem;
    MenuEditGroup: TMenuItem;
    Panel1: TPanel;
    TabGroups: TTabControl;
    TabDefault: TTabItem;
    MenuAddItem: TMenuItem;
    MenuDelItems: TMenuItem;
    MenuItemProp: TMenuItem;
    GroupSwitch: TSwitch;
    MenuCancel: TMenuItem;
    ItemGrid: TGrid;
    colCheck: TCheckColumn;
    colItemId: TStringColumn;
    colVal: TStringColumn;
    colQ: TStringColumn;
    MenuItem1: TMenuItem;
    MenuItemChart: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem6: TMenuItem;
    cIsActive: TCheckBox;
    MenuItems: TMenuItem;
    MenuItem3: TMenuItem;
    bUp: TSpeedButton;
    bDown: TSpeedButton;
    bProp: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure MenuDelItemsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MenuAddItemClick(Sender: TObject);
    procedure MenuEditGroupClick(Sender: TObject);
    procedure MenuAddGroupClick(Sender: TObject);
    procedure OnGroupsUpdate(Sender: TObject);
    procedure MenuDelGroupClick(Sender: TObject);
    procedure MenuCancelClick(Sender: TObject);
    procedure GroupSwitchSwitch(Sender: TObject);
    procedure ItemGridGetValue(Sender: TObject; const Col, Row: Integer; var Value: TValue);
    procedure TabGroupsChange(Sender: TObject);
    procedure MenuItemPropClick(Sender: TObject);
    procedure bInfoClick(Sender: TObject);
    procedure MenuItem1Click(Sender: TObject);
    procedure ItemGridClick(Sender: TObject);
    procedure ItemGridDblClick(Sender: TObject);
    procedure MenuItemChartClick(Sender: TObject);
    procedure cIsActiveChange(Sender: TObject);
    procedure bUpClick(Sender: TObject);
    procedure bDownClick(Sender: TObject);
    procedure bPropClick(Sender: TObject);
  private
    fDatachanges    : integer;
    fStartDataChange: TDateTime;
    fChangeActive : boolean;
    fActualGroup : TdOPCGroup;
    fServerItem: TServerItem;
    function makeTab(tc: TTabControl; Caption: string): TTabItem;
    procedure UpdateGroupItems(G: TdOPCGroup);
    function GetSelectedGroup: TdOPCGroup;
    procedure OnAddItems(Sender: TObject);
    procedure OnOPCGroupDatachange(Sender: TObject; ItemList: TdOPCItemList);
    procedure ShowEnabled(Enabled: boolean);
  public
    procedure ShowGroups(ServerItem: TServerItem);
  end;

var
  GroupsForm: TGroupsForm;

implementation

{$R *.fmx}

uses ItemPropDlg, GroupEditDlg;


function ToBool(BoolValue: boolean): string;
begin
  if BoolValue then result := 'true'  else  result := 'false'
end;

procedure TGroupsForm.cIsActiveChange(Sender: TObject);
var
  G : TdOPCGroup;
begin
  G := GetSelectedGroup;
  if G <> nil then
  begin
    if cIsActive.IsChecked and not fServerItem.Server.Active then
      ShowMessage('Not connected to OPC Server')
    else
    begin
      _WaitFor('Activate Group...',
      procedure
      begin
        G.IsActive := cIsActive.IsChecked;
        if G.IsActive and fServerItem.Server.Active then
          ShowEnabled(true)
        else
        begin
          ChartForm.Clear;
          ShowEnabled(false);
        end;
      end);
    end;
  end;
end;


procedure TGroupsForm.bInfoClick(Sender: TObject);
var
  G    : TdOPCGroup;
  Info : string;
  Secs  : double;
begin
  if fActualGroup <> nil then
  begin
    Secs := (Now-fStartDataChange) *SecsPerDay;
    G := fActualGroup;
    Info := 'OPC Server : '+ fServerItem.Server.Caption+ LineFeed+
            'Connected  : '+ ToBool(fServerItem.Server.Active)+ LineFeed+
            'Groupname  : '+ fActualGroup.Name+ LineFeed+
            'Is Active  : '+ ToBool(fActualGroup.IsActive)+ LineFeed+
            'Item count : '+ IntToStr(G.OPCItems.Count)+ LineFeed+
            'Datachanges: '+ IntToStr(fDatachanges)+ LineFeed+
            'Changes/sec: '+ Format('%.2f',[fDatachanges/Secs]);
    ShowMessage(Info);
  end;
end;

procedure TGroupsForm.bPropClick(Sender: TObject);
begin
  MenuItemPropClick(Self);
end;

procedure TGroupsForm.FormCreate(Sender: TObject);
begin
  inherited;
  AddPopup('Group && Items',PopupMenu1);
  fActualGroup := nil;
  fChangeActive := false;
  ItemGrid.StyledSettings := ItemGrid.StyledSettings - [TStyledSetting.Size];
  ItemGrid.TextSettings.Font.Size := cIsActive.TextSettings.Font.Size;

end;

procedure TGroupsForm.FormResize(Sender: TObject);
var
  W : extended;
  Delta: integer;
begin
  {$IFDEF NEXTGEN} Delta := 0; {$ELSE} Delta := 20;{$ENDIF}
  if MultiView1.Mode =  TMultiViewMode.Panel then
    W := Width- MultiView1.Width - ColQ.Width
  else
    W := Width - ColQ.Width;
  // W := Width - ColQ.Width;
  ColItemId.Width := W / 2 - Delta;
  ColVal.Width    := W / 2 - Delta;
  inherited;
end;

procedure TGroupsForm.MenuAddGroupClick(Sender: TObject);
begin
  ShowNewForm(GroupEditForm,TGroupEditForm,OnGroupsUpdate);
  GroupEditForm.AddGroup(fServerItem);
end;

procedure TGroupsForm.MenuAddItemClick(Sender: TObject);
begin
  if fServerItem.FServer.OPCGroups.Count > 0 then
  begin
    ShowNewForm(BrowseForm,TBrowseForm,OnAddItems);
    BrowseForm.ShowBrowseDlg(fServerItem,true);
  end
  else
    ShowMessage('First add a new group');
end;

procedure TGroupsForm.MenuCancelClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TGroupsForm.OnOPCGroupDatachange(Sender: TObject; ItemList: TdOPCItemList);
var
  OPCItem: TdOPCItem;
begin
  inc(fDatachanges);
  for OPCItem in Itemlist do
  begin
    ColVal.UpdateCell(OPCItem.Index);
    ColQ.UpdateCell(OPCItem.Index);
    if Assigned(ChartForm) and (ChartForm.OPCItem = OPCItem) then
      ChartForm.AddValue(OPCItem);
  end;
end;

procedure TGroupsForm.UpdateGroupItems(G : TdOPCGroup);
begin
  GroupSwitch.OnSwitch := nil;
  cIsActive.OnChange   := nil;
  if fActualGroup <>  nil then
   fActualGroup.OnDatachange := nil;

  fActualGroup := G;
  if G <> nil then
  begin
    GroupSwitch.IsChecked := G.IsActive;
    cIsActive.IsChecked   := G.IsActive;
    ItemGrid.RowCount := G.OPCItems.Count;
    if G.IsActive and fServerItem.Server.Active then
      ShowEnabled(true)
    else
      ShowEnabled(false);
    G.OnDatachange := OnOPCGroupDatachange;
  end
  else
  begin
   ItemGrid.RowCount := 0;
   GroupSwitch.IsChecked := false;
   cIsActive.IsChecked   := false;
  end;
  if ItemGrid.RowCount > 0 then
    ItemGrid.Selected := 0
  else
    ItemGrid.Selected := -1;
  GroupSwitch.OnSwitch := GroupSwitchSwitch;
  cIsActive.OnChange   := cIsActiveChange;
end;


procedure TGroupsForm.MenuDelGroupClick(Sender: TObject);
var
  Tab: TTabItem;
  G : tdOPCGroup;
begin
  Tab := TabGroups.ActiveTab;
  if (Tab <> nil) then
    IfYes('Delete selected group',
         procedure
         begin
            UpdateGroupItems(nil);
            TabGroups.ActiveTab := nil;
            G := TdOPCGroup(Tab.Tag);
            fServerItem.Server.OPCGroups.Remove(G.Name);
            Tab.DisposeOf;
            if TabGroups.TabCount > 0 then
            begin
               TabGroups.ActiveTab := TabGroups.Tabs[0];
               UpdateGroupItems(TdOPCGroup(TabGroups.ActiveTab.Tag));
            end;
         end);
end;


procedure TGroupsForm.MenuDelItemsClick(Sender: TObject);
begin
  if fActualGroup <> nil then
  begin
    IfYes('Delete all item?',
        procedure
        begin
           fActualGroup.OPCItems.RemoveAll;
           UpdateGroupItems(fActualGroup);
        end);
  end;
end;

procedure TGroupsForm.MenuEditGroupClick(Sender: TObject);
begin
  if TabGroups.ActiveTab = nil then
    exit;
  ShowNewForm(GroupEditForm,TGroupEditForm,OnGroupsUpdate);
  GroupEditForm.EditGroup(fServerItem,TabGroups.ActiveTab.Text);
end;

procedure TGroupsForm.MenuItem1Click(Sender: TObject);
var
  Idx : integer;
begin
  if fActualGroup = nil then
    exit;
  Idx := ItemGrid.Selected;
  if Idx >= 0 then
  begin
    ShowNewForm(WriteForm,TWriteForm);
    WriteForm.WriteItem(fServerItem.Server, fActualGroup, fActualGroup.OPCItems[Idx].ItemId);
  end;
end;

procedure TGroupsForm.MenuItemChartClick(Sender: TObject);
var
  Idx : integer;
begin
  if fActualGroup = nil then
    exit;
  Idx := ItemGrid.Selected;
  if Idx >= 0 then
  begin
    ShowNewForm(ChartForm,TChartForm);
    ChartForm.ShowChartValues(fServerItem.Server, fActualGroup.OPCItems[Idx]);
  end;
end;

procedure TGroupsForm.MenuItemPropClick(Sender: TObject);
var
  Index : integer;
begin
  if fActualGroup = nil then
    exit;
  Index := ItemGrid.Selected;
  if index >= 0 then
  begin
    ShowNewForm(ItemPropForm,TItemPropForm);
    ItemPropForm.ShowProperties(fServerItem.Server, fActualGroup.OPCItems[Index].ItemId);
  end;
end;

function TGroupsForm.makeTab(tc : TTabControl; Caption: string): TTabItem;
var
  tab: TTabItem;
begin
  tab := TTabItem.Create(tc);
  tab.text := Caption;
  tab.Parent := tc;
  result := tab;
end;

function TGroupsForm.GetSelectedGroup: TdOPCGroup;
begin
  result := nil;
  if TabGroups.ActiveTab <> nil then
    result := TdOPCGroup(TabGroups.ActiveTab.Tag);
end;

procedure TGroupsForm.ItemGridClick(Sender: TObject);
var
  G    : TdOPCGroup;
  Item : TdOPCItem;
  Index: integer;
begin
  if ColCheck.Visible then
  begin
    Index := ItemGrid.Selected;
    G := GetSelectedGroup;
    if (G <> nil) and (Index >= 0) then
    begin
      Item := G.OPCItems[Index];
      if Item.Tag = 0 then
        Item.Tag := 1
      else
        Item.Tag := 0;
    end;
    ColCheck.UpdateCell(Index);
  end;
end;

procedure TGroupsForm.ItemGridDblClick(Sender: TObject);
var
  G    : TdOPCGroup;
  Item : TdOPCItem;
  Index: integer;
begin
  if fActualGroup = nil then exit;
  IfYes('Delete Item?', procedure
    begin
      if fActualGroup.OPCItems.Count > 0 then
      begin
        Index := ItemGrid.Selected;
        G := GetSelectedGroup;
        Item := G.OPCItems[Index];
        Item.Free;
        UpdateGroupItems(G);
      end;
    end);
end;

procedure TGroupsForm.ItemGridGetValue(Sender: TObject; const Col, Row: Integer; var Value: TValue);
var
  OPCItem : TdOPCItem;
begin
  if fActualGroup = nil then exit;
  if Row <= fActualGroup.OPCItems.Count then
  begin
    OPCItem := fActualGroup.OPCItems[Row];
    if Col = colCheck.Index  then Value := OPCItem.Tag = 1 else
    if Col = ColItemId.Index then Value := OPCItem.ItemName else
    if Col = ColQ.Index      then Value := OPCItem.QualityAsString else
    if Col = ColVal.Index    then Value := OPCItem.ValueStr;
  end;
end;

procedure TGroupsForm.OnAddItems(Sender: TObject);
var
  G : TdOPCGroup;
begin
  G := GetSelectedGroup;
  if G <> nil then
  begin
    _WaitFor('Adding items...',
    procedure
    var
     Item : string;
    begin
      G.OPCItems.BeginUpdate;
      try
        for Item in BrowseForm.OPCItemIds do
         G.OPCItems.AddItem(Item);
        G.OPCItems.OPCAddAllItems;
      finally
        G.OPCItems.EndUpdate;
        UpdateGroupItems(G);
      end;
    end);
  end;
end;

procedure TGroupsForm.OnGroupsUpdate(Sender: TObject);
var
  Tab: TTabItem;
  G: TdOPCGroup;
  i : integer;
begin
  TabGroups.BeginUpdate;
  fChangeActive := false;
  try
    TabGroups.ActiveTab := nil;
    for i := TabGroups.TabCount downto 0 do
      TabGroups.Tabs[i].DisposeOf;

    for G in fServerItem.Server.OPCGroups do
    begin
      Tab := makeTab(TabGroups, G.Name);
      Tab.Tag := NativeInt(G);
    end;

    if (TabGroups.ActiveTab = nil) and (TabGroups.TabCount > 0) then
      TabGroups.ActiveTab := TabGroups.Tabs[0];
  finally
    TabGroups.EndUpdate;
    fChangeActive := true;
  end;
  UpdateGroupItems(GetSelectedGroup);
end;

procedure TGroupsForm.ShowEnabled(Enabled: boolean);
begin
   if Enabled then
     ItemGrid.Opacity := 1
   else
     ItemGrid.Opacity := 0.5;
end;

procedure TGroupsForm.ShowGroups(ServerItem: TServerItem);
begin
  fDatachanges     := 0;
  fStartDataChange := Now;
  fChangeActive    := false;
  fServerItem      := ServerItem;
  fServerItem.Server.OPCGroupDefault.IsActive := false;
  OnGroupsUpdate(nil);
end;


procedure TGroupsForm.bUpClick(Sender: TObject);
var
  SelIndex : integer;
begin
  SelIndex := ItemGrid.Selected;
  if SelIndex > 0 then
  begin
    Dec(SelIndex);
    ItemGrid.SelectRow(SelIndex);
  end;
end;

procedure TGroupsForm.bDownClick(Sender: TObject);
var
  SelIndex : integer;
begin
  SelIndex := ItemGrid.Selected;
  if SelIndex < ItemGrid.RowCount-1 then
  begin
    inc(SelIndex);
    ItemGrid.SelectRow(SelIndex);
  end;
end;

procedure TGroupsForm.GroupSwitchSwitch(Sender: TObject);
var
  G : TdOPCGroup;
begin
  G := GetSelectedGroup;
  if G <> nil then
  begin
    if GroupSwitch.IsChecked and not fServerItem.Server.Active then
      ShowMessage('Not connected to OPC Server');
    try
      ShowWaitForm('Activate Group');
      G.IsActive := GroupSwitch.IsChecked;
    finally
      HideWaitForm;
      if G.IsActive and fServerItem.Server.Active then
        ShowEnabled(true)
      else
      begin
        ChartForm.Clear;
        ShowEnabled(false);
      end;
    end;
  end;
end;

(*
procedure TGroupsForm.GroupSwitchSwitch(Sender: TObject);
var
  G : TdOPCGroup;
begin
  G := GetSelectedGroup;
  if G <> nil then
  begin
    if GroupSwitch.IsChecked and not fServerItem.Server.Active then
      ShowMessage('Not connected to OPC Server');
    TTask.Run(
      procedure
      begin
        G.IsActive := GroupSwitch.IsChecked;
        TThread.Synchronize(nil,
          procedure
          begin
            if G.IsActive and fServerItem.Server.Active then
              ShowEnabled(true)
            else
            begin
              ChartForm.Clear;
              ShowEnabled(false);
            end;
          end
       );
      end);
  end;
end;
*)


procedure TGroupsForm.TabGroupsChange(Sender: TObject);
begin
  if fChangeActive then
  begin
    if TabGroups.ActiveTab = nil then
      UpdateGroupItems(nil)
    else
      UpdateGroupItems(TdOPCGroup(TabGroups.ActiveTab.Tag));
  end;
end;

end.
