unit ItemPropDlg;

interface

uses
  WriteDlg, dopcda, dopccomn, dOPCIntf, FMX.Header, ua.dtstr,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMX.Menus,
  FMX.ListView.Types, FMX.ListView, System.Rtti, FMX.Grid, ServerData,
  FMX.Controls.Presentation, FMX.MultiView, FMX.ScrollBox, FMX.Grid.Style;

type
  TItemPropForm = class(TCoreForm)
    PopupMenu1: TPopupMenu;
    MenuBack: TMenuItem;
    MenuItemWrite: TMenuItem;
    PropGrid: TStringGrid;
    ColProp: TStringColumn;
    ColVals: TStringColumn;
    bWrite: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure MenuBackClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MenuItemWriteClick(Sender: TObject);
  private
    fRow : integer;
    fItemId: string;
    fOPCServer : TOPCDAServer;
    procedure AddProp(const Prop, Value: string);
    procedure AddCOMPropsToList(Props: TdOPCItemProperties);
    procedure AddUAPropsToList(Props: TdOPCItemProperties);
  public
    procedure ShowProperties(OPCServer  : TOPCDAServer; ItemId: string);
  end;

var
  ItemPropForm: TItemPropForm;

implementation

{$R *.fmx}

type
  TCustomGridHelper = class helper for TCustomGrid
  public
    procedure AdjustStretchColumn(ACol: Integer);
  end;


procedure TCustomGridHelper.AdjustStretchColumn(ACol: Integer);
var
  I: Integer;
  W: Single;
begin
  W := 5 + VScrollBar.Width;
  for I := 0 to ColumnCount - 1 do begin
    if I = ACol then Continue;
    if Columns[I].Visible then
      W := W + Columns[I].Width;
  end;
  Columns[ACol].Width := Width - W;
end;


procedure TItemPropForm.FormCreate(Sender: TObject);
begin
  inherited;
  fRow := 0;
  AddPopup('Server Status',PopupMenu1);
end;

procedure TItemPropForm.FormResize(Sender: TObject);
Var
  W : extended;
begin
  inherited;
   if MultiView1.Mode =  TMultiViewMode.Panel then
    W := Width- MultiView1.Width
  else
    W := Width;
  PropGrid.Columns[1].Width := W - PropGrid.Columns[0].Width;
end;

procedure TItemPropForm.MenuBackClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TItemPropForm.MenuItemWriteClick(Sender: TObject);
begin
  ShowNewForm(WriteForm,TWriteForm);
  WriteForm.WriteItem(fOPCServer, nil,fItemId);
end;

procedure TItemPropForm.AddProp(const Prop: string; const Value: string);
begin
  PropGrid.Cells[0,fRow] := Prop;
  PropGrid.Cells[1,fRow] := Value;
  Inc(fRow);
end;

procedure TItemPropForm.AddUAPropsToList(Props:TdOPCItemProperties);
var
 Prop     : TdOPCItemProperty;
begin
  for Prop in Props do
  begin
     case Prop.Id of
      UAOPC_PROP_CDT     : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+uaDataTypeToStr(Prop.Value)+')');
      UAOPC_PROP_QUALITY : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+QualitytoStr(Prop.Value)+')');
      UAOPC_PROP_URIGHTS : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+AccessRightToStr(Prop.Value)+')');
      else AddProp(Prop.Description, VarToAStr(Prop.Value));
     end;
  end;
end;

procedure TItemPropForm.AddCOMPropsToList(Props:TdOPCItemProperties);
var
 Prop     : TdOPCItemProperty;
begin
  for Prop in Props do
  begin
     case Prop.Id of
      OPC_PROP_CDT     : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+VarTypeAsText(Prop.Value)+')');
      OPC_PROP_QUALITY : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+QualitytoStr(Prop.Value)+')');
      OPC_PROP_RIGHTS  : AddProp(Prop.Description, VarToAStr(Prop.Value)+' ('+AccessRightToStr(Prop.Value)+')');
      else AddProp(Prop.Description, VarToAStr(Prop.Value));
     end;
  end;
end;


procedure TItemPropForm.ShowProperties(OPCServer  : TOPCDAServer; ItemId: string);
var
 Props    : TdOPCItemProperties;
begin
  fRow := 0;
  ColProp.BeginUpdate;
  ColVals.BeginUpdate;
  try
    fOPCServer := OPCServer;
    if fOPCServer.Protocol <> coUA then
      AddProp('Item Id', ItemId);
    fItemId := ItemId;
    if OPCServer.Active then
    begin
      ShowWaitForm('Get Properties...');
      try
        Props := TdOPCItemProperties.create(OPCServer,ItemId);
        PropGrid.RowCount := Props.Count+1;
        if OPCServer.Protocol = coUA then
          AddUAPropsToList(Props)
        else
          AddCOMPropsToList(Props);
        Props.Free;
      finally
        HideWaitForm;
      end;
    end
    else
      AddProp('State','not connected');
    PropGrid.Columns[1].Width := Width - PropGrid.Columns[0].Width;
    PropGrid.RowCount := fRow;
    //PropGrid.AdjustStretchColumn(1);
  finally
    ColProp.EndUpdate;
    ColVals.EndUpdate;
  end;
end;


end.
