unit MainDlg;

interface

uses
  GroupEditDlg, dOPCIntf, Threading, WaitDlg, FMX.Ani,
  CoreDlg, FMX.Controls, FMX.Menus, FMX.Types, FMX.ListBox, FMX.Layouts,
  FMX.MultiView, FMX.StdCtrls, System.Classes, FMX.Controls.Presentation,
  ServerData, ServerStatusDlg, GroupsDlg, PlatformStuff,
  System.SysUtils, System.Types, System.UITypes, System.Variants,
  FMX.Objects,  FMX.Graphics, FMX.Forms, FMX.Dialogs,
  System.IOUtils, FMX.Gestures, System.ImageList;

type
  TOPCUAExplorer = class(TCoreForm)
    ListBoxServers: TListBox;
    ListBoxItem1: TListBoxItem;
    PopupServer: TPopupMenu;
    MenuServerAdd: TMenuItem;
    MenuServerEdit: TMenuItem;
    MenuServerDel: TMenuItem;
    MenuServerConnect: TMenuItem;
    MenuServerDisconnect: TMenuItem;
    MenuServerStatus: TMenuItem;
    MenuServerItems: TMenuItem;
    MenuSaveData: TMenuItem;
    StyleBook: TStyleBook;
    MenuServerBrowse: TMenuItem;
    bProp: TSpeedButton;
    bDisconnect: TSpeedButton;
    bConnect: TSpeedButton;
    bBrowse: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure MenuServerAddClick(Sender: TObject);
    procedure MenuServerDelClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuServerEditClick(Sender: TObject);
    procedure ListBoxServersGesture(Sender: TObject; const EventInfo: TGestureEventInfo; var Handled: Boolean);
    procedure ListBoxServersDblClick(Sender: TObject);
    procedure MenuServerConnectClick(Sender: TObject);
    procedure MenuServerDisconnectClick(Sender: TObject);
    procedure MenuServerStatusClick(Sender: TObject);
    procedure bInfoClick(Sender: TObject);
    procedure MenuServerItemsClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure MenuSaveDataClick(Sender: TObject);
    procedure bBackClick(Sender: TObject);
    procedure MenuServerBrowseClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    fBMPConnect    : TBitmap;
    fBMPDisConnect : TBitmap;
    fServerData    : TOPCExplorerData; // stores all OPC Server Data
    fDataFileName  : string;
    procedure LoadOPCServerData;
    function  GetServerItem(Info: boolean): TServerItem;
    procedure ServerConnect(Action: integer);
    procedure OnAddServer(Sender : TObject);
    procedure OnShutdown(Sender: TObject; Reason: string);
    procedure OnConnect(Sender: TObject);
    procedure OnDisconnect(Sender: TObject);
    procedure UpdateServerStateBMPs;
    procedure ShowGroupDlg;
    procedure AddListItem(Listbox: TListBox; Bitmap: TBitmap; Item: TServerItem);
  public
  end;

var
  OPCUAExplorer: TOPCUAExplorer;

implementation

{$R *.fmx}

uses AboutDlg, ServerAddDlg, BrowseDlg;

type
 TMyListItem = TListBoxItem;

function GetServerListItem(Listbox: TListBox; Item: TServerItem) : TListBoxItem;
var
  i     : Integer;
  LItem : TListBoxItem;
begin
  result := nil;
  for i := 0 to Listbox.Count-1 do
  begin
    LItem := Listbox.ItemByIndex(i);
    if LItem.Data = Item  then
    begin
      result := LItem;
      break;
    end;
  end;
end;

//procedure TOPCUAExplorer.OnListMouseEnter(Item: TObject);
//var
//  LItem : TListboxItem;
//begin
//  LItem := TListboxItem(Item);
//  LItem.TextSettings.Font.Style := LItem.TextSettings.Font.Style + [TFontStyle.fsUnderline];
//end;
//
//
//procedure TOPCUAExplorer.OnListMouseLeave(Item: TObject);
//var
//  LItem : TListboxItem;
//begin
//  LItem := TListboxItem(Item);
//  LItem.TextSettings.Font.Style := LItem.TextSettings.Font.Style - [TFontStyle.fsUnderline];
//end;

procedure TOPCUAExplorer.AddListItem(Listbox: TListBox; Bitmap: TBitmap; Item: TServerItem);
var
  ListItem : TMyListItem;
begin
   ListBox.BeginUpdate;
   try
     ListItem := TMyListItem.Create(ListBox);
     ListItem.Data               := Item;
     ListItem.Text               := Item.Server.Caption;
     ListItem.Height             := 48;
     ListItem.ItemData.Accessory := TListBoxItemData.TAccessory.aNone;
     ListItem.ItemData.Detail    := Item.Server.ServerName;
     ListItem.ItemData.Bitmap.Assign(Bitmap);
     ListItem.StyleLookup        := 'listboxitembottomdetail';
     ListItem.TextAlign          := TTextAlign.Center;
//     {$IFNDEF NEXTGEN}
//     ListItem.OnMouseEnter       := OnListMouseEnter;
//     ListItem.OnMouseLeave       := OnListMouseLeave;
//     ListItem.HitTest := true;
//     {$ENDIF}
     ListBox.AddObject(Listitem);
   finally
     Listbox.EndUpdate;
   end;
end;


procedure TOPCUAExplorer.FormDestroy(Sender: TObject);
begin
  fServerData.Closeall;
  fServerData.Free;
  fBMPConnect.Free;
  fBMPDisConnect.Free;
  inherited;
end;


procedure TOPCUAExplorer.FormCreate(Sender: TObject);
begin
  inherited;
  fBMPConnect      := CreateBMPFromResource('CONNECT');
  fBMPDisConnect   := CreateBMPFromResource('DISCONNECT');
  AddPopup('Server',PopupServer);
  fServerData := TOPCExplorerData.create(nil);
  LoadOPCServerData;
  {$IFNDEF NEXTGEN}   // on Windows and MAC
   if fServerData.top > TOPCExplorerData.posUndef then
   begin
    Position :=  TFormPosition.Designed;
    SetBounds(fServerData.left,fServerData.top,fServerData.width,fServerData.height);
   end
   else
   begin   // set default Desktop width
     width := Round(Screen.Size.Width) div 3;
     left  := width;
   end;
  {$ENDIF}
end;

// handler if user clicks on the menu button
procedure TOPCUAExplorer.bBackClick(Sender: TObject);
begin
  // inherited;
  MenuSaveDataClick(Self);
end;

procedure TOPCUAExplorer.bInfoClick(Sender: TObject);
begin
 ShowNewForm(AboutForm,TAboutForm);
end;

procedure TOPCUAExplorer.UpdateServerStateBMPs;
var
  i     : Integer;
  LItem : TListBoxItem;
  SItem : TServerItem;
begin
  ListBoxServers.BeginUpdate;
  try
    for i := 0 to ListBoxServers.Count-1 do
    begin
      LItem := ListBoxServers.ItemByIndex(i);
      SItem := TServerItem(LItem.Data);
      if SItem.Server.Active then
        LItem.ItemData.Bitmap.Assign(fBMPConnect)
      else
        LItem.ItemData.Bitmap.Assign(fBMPDisConnect);
    end;
  finally
    ListBoxServers.EndUpdate;
  end;
  ListBoxServers.ApplyStyleLookup;
  ListBoxServers.RealignContent;
end;

// Action = 0 disconnect
// Action = 1 connect
// Action = 3 troggle
procedure TOPCUAExplorer.ServerConnect(Action: integer);
var
  ListBoxItem : TListBoxItem;
  SItem  : TServerItem;
begin
  ListBoxItem := ListBoxServers.Selected;
  if ListBoxItem <> nil then
  begin
    SItem  := TServerItem(ListBoxItem.Data);
    if Action = 3 then   // troggle
    begin
       Action := 1;
       if SItem.Server.Active then
         Action := 0;
    end;
    if Action = 0 then  // if disconnect
    begin
      _WaitFor('Disconnecting...',procedure begin SItem.Server.Active := false;  end);
    end
    else
    begin
      SItem.Server.OnServerShutdown := OnShutdown;
      SItem.Server.OnDisconnect     := OnDisconnect;
      SItem.Server.OnConnect        := OnConnect;
      SItem.Server.SetParam('proxy',SItem.Server.Proxy);
      _WaitFor('Connecting...',
      procedure
      begin
        SItem.Server.Active := true;
      end);
    end;
  end
  else
    ShowMessage('Please select a server');
end;

procedure TOPCUAExplorer.ListBoxServersDblClick(Sender: TObject);
begin
  ServerConnect(3);
end;

procedure TOPCUAExplorer.ListBoxServersGesture(Sender: TObject;
  const EventInfo: TGestureEventInfo; var Handled: Boolean);
begin
  inherited;
  if Eventinfo.GestureID = igiLongTap then // if you trap long on server item -> connect to server
    ListBoxServersDblClick(self)
end;

procedure TOPCUAExplorer.LoadOPCServerData;
var
  i            : integer;
  SItem        : TServerItem;
  AddDefault   : boolean;
begin
  AddDefault := false;
  ListboxServers.Clear;
  fDataFileName := TPath.Combine(TPath.GetDocumentsPath,'dOPC_UA.dat');
  if FileExists(fDataFileName) then
  begin
     try
       fServerData.LoadFromFile(fDataFileName);
       for i := 0 to fServerData.Serverlist.Count -1 do
          AddListItem(ListBoxServers,fBMPDisConnect,fServerData.Serverlist[i]);
     except
       On E: Exception do
       begin
         ifYes ('Can not read configuration file: '+Linefeed+
                 fDataFileName+Linefeed+
                 'Wrong file format or older version'+Linefeed+
                 E.Message+Linefeed+'Delete File?', procedure
           begin
             DelFile(fDataFileName);
             fServerData.ServerList.Clear;
             AddDefault := true;
           end);
       end;
     end;
  end
  else
    AddDefault := true;

  if AddDefault then
  begin
    SItem := fServerData.ServerList.Add;
    SItem.Server.Caption := 'Public OPC UA Server';
    SItem.Server.ServerName := 'opc.tcp://opcua.demo-this.com:51210/ua/sampleserver';
    SItem.Server.Protocol := coUA;
    AddListItem(ListBoxServers,fBMPDisConnect,SItem);
    SItem := fServerData.ServerList.Add;
    SItem.Server.Caption := 'Public OPC XML DA Server';
    SItem.Server.ServerName := 'http://opcxml.demo-this.com/XmlDaSampleServer/Service.asmx';
    SItem.Server.Protocol := coXML;
    AddListItem(ListBoxServers,fBMPDisConnect,SItem);
    SItem := fServerData.ServerList.Add;
    SItem.Server.Caption := 'dOPC XGate Kassl Test Server';
    SItem.Server.ServerName := 'http://localhost/opcxmlda/dOPCSim.Kassl.Simulation';
    SItem.Server.Protocol := coXML;
    AddListItem(ListBoxServers,fBMPDisConnect,SItem);
  end;
  if ListBoxServers.Count > 0  then
    ListBoxServers.ItemIndex := 0;
end;

// save server item with groups and items to the configuration file
procedure TOPCUAExplorer.MenuSaveDataClick(Sender: TObject);
begin
 if fServerData.IsOneServerConnected then
 begin
   ShowMessage('You have to disconnect all server first');
  exit;
 end;
 fServerData.left  := left;
 fServerData.top   := top;
 fServerData.width := width;
 fServerData.height:= height;
 fServerData.CloseAll;
 fServerData.SaveToFile(fDataFileName);  // now save server data
 Close;
end;

procedure TOPCUAExplorer.MenuServerAddClick(Sender: TObject);
begin
 ShowNewForm(AddServerForm,TAddServerForm,OnAddServer); // call new dialog
 AddServerForm.AddServer(GetServerItem(false));   // gives data of the selected server to the dialog
end;

procedure TOPCUAExplorer.MenuServerBrowseClick(Sender: TObject);
var
  SItem : TServeritem;
begin
  SItem := GetServerItem(true);
  if SItem <> nil then
  begin
    ShowNewForm(BrowseForm, TBrowseForm);
    BrowseForm.ShowBrowseDlg(SItem,false);
  end;
end;

// user click OK button an want to add or change server item
procedure TOPCUAExplorer.OnAddServer(Sender: TObject);
var
  SItem : TServerItem;
  LItem : TListBoxItem;
begin
  SItem := AddServerForm.fEditServerItem;
  if AddServerForm.fEditServerItem = nil then
    SItem := fServerData.ServerList.Add;
  SItem.Server.ClientName := 'dOPC UA Mobile Explorer';
  SItem.Server.Caption    := AddServerForm.eServerName.Text;
  SItem.Server.ServerName := AddServerForm.eServerURL.Text;
  SItem.Server.Proxy      := AddServerForm.eProxy.Text;
  SItem.Server.DetectProtocol;  // Set Protocol
  if AddServerForm.fEditServerItem <> nil then
  begin  // may be user has change name or url ->
         // we have to change also the listbox item
    LItem := GetServerListItem(ListBoxServers,SItem);
    if LItem <> nil then
    begin
      LItem.Text :=  SItem.Server.Caption;
      LItem.ItemData.Detail := SItem.Server.ServerName;
      LItem.IsSelected := true;
    end
    else
      ShowMessage('error in server list');
  end
  else // new server item -> add to list box
     AddListItem(ListBoxServers,fBMPDisConnect,SItem);
end;

procedure TOPCUAExplorer.OnConnect(Sender: TObject);
begin
  UpdateServerStateBMPs;
end;

procedure TOPCUAExplorer.OnDisconnect;
begin
  UpdateServerStateBMPs;
end;

procedure TOPCUAExplorer.OnShutdown(Sender: TObject; Reason: string);
begin
  ShowMessage('Shutdown from server '+reason);
end;

procedure TOPCUAExplorer.MenuServerConnectClick(Sender: TObject);
begin
  ServerConnect(1);
end;

function TOPCUAExplorer.GetServerItem(Info: boolean): TServerItem;
var
 ListBoxItem : TListBoxItem;
begin
  result := nil;
  ListBoxItem := ListBoxServers.Selected;
  if ListBoxItem <> nil then
  begin
    if ListBoxItem.Data <> nil then
      result := TServerItem(ListBoxItem.Data);
  end;
  if (result = nil) and (Info = true) then
    ShowMessage('Please select a server');
end;

procedure TOPCUAExplorer.MenuServerDelClick(Sender: TObject);
var
 ListBoxItem : TListBoxItem;
begin
  ListBoxItem := ListBoxServers.Selected;
  if ListBoxItem <> nil then
  begin
    if (ListBoxItem.Data <> nil) then
      IfYes('Delete: '+TServerItem(ListBoxItem.Data).Server.Caption, procedure
       begin
        fServerData.ServerList.DeleteServer(TServerItem(ListBoxItem.Data));
        ListBoxItem.DisposeOf;
       end);
  end
  else ShowMessage('first select server');
end;

procedure TOPCUAExplorer.MenuServerDisconnectClick(Sender: TObject);
var
  Server : TServerItem;
begin
  Server := GetServerItem(false);
  if (Server <> nil) and (Server.FServer.IsActive) then
   _WaitFor('Disconnecting...', procedure begin Server.FServer.Active := false; end);
end;

procedure TOPCUAExplorer.MenuServerEditClick(Sender: TObject);
var
  SItem : TServerItem;
begin
  SItem := GetServerItem(true);
  if SItem <> nil then
  begin
    if not SItem.Server.Active then
    begin
      ShowNewForm(AddServerForm,TAddServerForm,OnAddServer);
      AddServerForm.EditServer(SItem);
    end
    else
      ShowMessage('first disconnect server');
  end
end;

procedure TOPCUAExplorer.Button1Click(Sender: TObject);
begin
  // only for testing
end;

procedure TOPCUAExplorer.ShowGroupDlg;
var
  SItem : TServeritem;
begin
  SItem := GetServerItem(true);
  if SItem = nil then exit;
//  if AutoConnect then
//    if SItem.Server.Active = false then
//      ServerConnect(1);
  if SItem.Server.Active then
  begin
    ShowNewForm(GroupsForm,TGroupsForm);
    GroupsForm.ShowGroups(SItem);
  end
  else
    ShowMessage('first connect to server');
end;


procedure TOPCUAExplorer.SpeedButton1Click(Sender: TObject);
begin
  ShowMessage('Hallo');
end;

procedure TOPCUAExplorer.MenuServerItemsClick(Sender: TObject);
begin
  ShowGroupDlg;
end;

procedure TOPCUAExplorer.MenuServerStatusClick(Sender: TObject);
var
  SItem : TServeritem;
begin
  SItem := GetServerItem(true);
  if SItem <> nil then
  begin
    ShowNewForm(ServerStatusForm,TServerStatusForm);
    ServerStatusForm.ShowProperties(SItem);
  end;
end;

end.
