unit PlatformStuff;

interface

uses
  FMX.Styles, FMX.Dialogs, Types, FMX.Types, IOUtils, FMX.Controls, FMX.Graphics, Classes,
  sysutils;

procedure LaunchURL(const URL: string);
function  DelFile(const FileName: string): Boolean;
function  CreateBMPFromResource(BMPName: string): FMX.Graphics.TBitmap;
function  _LoadStyle(Path: string; StyleName: string = ''): boolean;
procedure ExitApp;


implementation

{$IFDEF Android}
uses
  Posix.UniStd,
  Androidapi.Helpers,
  Androidapi.JNI.App,
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.JavaTypes,
  Androidapi.JNI.Net;


function DelFile(const FileName: string): Boolean;
begin
  result := DeleteFile(FileName);
end;

procedure ExitApp;
begin
  TAndroidHelper.Activity.Finish;
end;

procedure LaunchURL(const URL: string);
var
  Intent: JIntent;
begin
  Intent := TJIntent.JavaClass.init(TJIntent.JavaClass.ACTION_VIEW,
    TJnet_Uri.JavaClass.parse(StringToJString(URL)));
  TAndroidHelper.Activity.startActivity(Intent);
end;
{$ENDIF}

{$IFDEF MSWINDOWS}
uses
  Winapi.ShellAPI, Winapi.Windows;

procedure LaunchURL(const URL: string);
begin
  ShellExecute(0, 'OPEN', PChar(URL), '', '', SW_SHOWNORMAL);
end;

function DelFile(const FileName: string): Boolean;
begin
  result := DeleteFile(PChar(FileName));
end;

procedure ExitApp;
begin
  Halt;
end;


{$ENDIF}


{$IFDEF MACOS}
uses
 Posix.UniStd,
{$IFDEF IOS}
  Macapi.Helpers, iOSapi.Foundation, iOSapi.UIKit;
{$ELSE}
{$IFDEF MACOS}
  Macapi.Helpers, Macapi.Foundation, Macapi.AppKit;
{$ENDIF MACOS}
{$ENDIF IOS}

function DelFile(const FileName: string): Boolean;
begin
  result := DeleteFile(FileName);
end;

procedure ExitApp;
begin
end;

procedure LaunchURL(const URL: string);
var
  U : NSUrl;
begin
  //U := TNSUrl.Wrap(TNSUrl.OCClass.URLWithString(NSStr(URL)));
   U := TNSUrl.Wrap(TNSUrl.OCClass.URLWithString(StrToNSStr(URL)));

  {$IFDEF IOS}
  TUIApplication.wrap(TUIApplication.OCClass.SharedApplication).openURL(U);
  {$ELSE}
  TNSWorkspace.wrap(TNSWorkspace.OCClass.sharedWorkspace).openURL(U);
  {$ENDIF}
end;
{$ENDIF}

function CreateBMPFromResource(BMPName: string): FMX.Graphics.TBitmap;
var
  S: TResourceStream;
begin
  // result := nil;
  if FindResource(HInstance, PChar(BMPName), RT_RCDATA) <> 0 then
  begin
    S := TResourceStream.Create(HInstance, BMPName, RT_RCDATA);
    S.Position := 0;
    Result := FMX.Graphics.TBitmap.Create;
    Result.LoadFromStream(S);
    S.Free;
  end
  else
    raise Exception.Create('BMP ' + BMPName + ' not found in recource file');
end;

function _LoadStyle(Path: string; StyleName: string): boolean;
var
  P: integer;
  Style: TFmxObject;
var
  FName: string;
begin
  Result := false;
  if StyleName = '' then // Load default style for the os
    StyleName := 'default.style';
  try
     FName := TPath.Combine(TPath.GetDocumentsPath,'dOPCMExStyles');
     FName := TPath.Combine(FName,StyleName);
     if FileExists(FName) then
     begin
       TStyleManager.SetStyleFromFile(FName);
       Result := true;
     end
     else
     begin
        P := pos('.', StyleName);
        if P > 0 then
          Delete(StyleName, P, Length(StyleName));
         Style := TStyleStreaming.LoadFromResource(HInstance, StyleName, RT_RCDATA);
         TStyleManager.SetStyle(Style);
         Result := true;
         // Result := TStyleManager.TrySetStyleFromResource(StyleName);
     end;
  except
      ShowMessage('can not load style: '+StyleName);
  end;
end;

end.
