unit WaitDlg;

interface

uses
  System.SysUtils, System.Classes,
  FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Objects, FMX.Controls.Presentation, FMX.Types;

type
  TWaitFrame = class(TFrame)
    Rectangle1: TRectangle;
    WaitPanel: TPanel;
    AniIndicator1: TAniIndicator;
    LMsg: TLabel;
  private
    FMessage: string;
    procedure SetMessage(const Value: string);
    property Msg: string read FMessage write SetMessage;
  public
    class procedure RunWaitDlg(Parent: TForm; Info: string; AThreadProc: TThreadProcedure);
    procedure ShowWait(Parent: TForm);
    procedure HideWait;
  end;

  var
    WaitFrame: TWaitFrame = nil;

implementation

{$R *.fmx}


class procedure TWaitFrame.RunWaitDlg(Parent: TForm; Info: string; AThreadProc: TThreadProcedure);
begin
  if not Assigned(WaitFrame) then
    WaitFrame := TWaitFrame.Create(Parent);
  WaitFrame.Parent := Parent;
  WaitFrame.Msg    := Info;
  WaitFrame.ShowWait(Parent);
  if assigned(AThreadProc) then
  begin
    TThread.CreateAnonymousThread(
      procedure
      var
        ErrorMsg: string;
      begin
        try
          ErrorMsg := '';
          AThreadProc;
        except
          on E: Exception do
            ErrorMsg := E.Message;
        end;
        TThread.Queue(nil,
        procedure
        begin
          WaitFrame.HideWait;
          if ErrorMsg <> '' then
            ShowMessage(ErrorMsg);
        end);
      end).Start;
  end;
end;

procedure TWaitFrame.SetMessage(const Value: string);
begin
  LMsg.Text := Value;
  FMessage := Value;
end;

procedure TWaitFrame.HideWait;
var
  C: TFmxObject;
begin
  Visible := false;
  for C in fParent.Children do
    if C is TControl then
     TControl(C).Enabled := true;
end;


procedure TWaitFrame.ShowWait(Parent: TForm);
var
  C: TFmxObject;
begin
  WaitFrame.Parent := Parent;
  if Visible = false then
  begin
    Visible := true;
    AniIndicator1.Visible := true;
    AniIndicator1.Enabled := true;
    Application.ProcessMessages;
    for C in fParent.Children do
      if (C is TControl) and (C <> WaitFrame) then
       TControl(C).Enabled := false;
  end;
end;

end.
