unit WriteDlg;

interface

uses
  ServerData, dopcda, dOPCComn, System.SysUtils, System.Types,
  System.UITypes, dOPCIntf, ua.dtstr,
  System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.Layouts, FMX.ListBox, FMX.Objects, FMX.Menus, FMX.Edit,
  FMX.Controls.Presentation, FMX.MultiView;

type
  TWriteForm = class(TCoreForm)
    VertScrollBox1: TVertScrollBox;
    PopupWrite: TPopupMenu;
    MenuOK: TMenuItem;
    MenuCancel: TMenuItem;
    LItemId: TLabel;
    eDatatType: TEdit;
    Label2: TLabel;
    Layout1: TLayout;
    Label3: TLabel;
    eNewValue: TEdit;
    ClearEditButton1: TClearEditButton;
    Label4: TLabel;
    eCurrentValue: TEdit;
    EditButton1: TEditButton;
    procedure MenuCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditButton1Click(Sender: TObject);
    procedure MenuOKClick(Sender: TObject);
  private
    fOPCGroup: TdOPCGroup;
    fOPCServer: TOPCDAServer;
    fOPCItem: TdOPCItem;
    fGroupDelete: boolean;
    procedure ReadCurrentValue;
  public
    procedure BackToParentForm(Ok: boolean = false); override;
    procedure WriteItem(OPCServer: TOPCDAServer; OPCGroup: TdOPCGroup;
      ItemId: string);
  end;

var
  WriteForm: TWriteForm;

implementation

{$R *.fmx}

procedure TWriteForm.BackToParentForm(Ok: boolean);
begin
  inherited;
  if fGroupDelete then
    fOPCGroup.DisposeOf;
end;

procedure TWriteForm.FormCreate(Sender: TObject);
begin
  inherited;
  AddPopup('Item Write',PopupWrite);
  fGroupDelete := false;
end;

procedure TWriteForm.MenuCancelClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TWriteForm.MenuOKClick(Sender: TObject);
begin
  EditButton1Click(self);
  BackToParentForm;
end;

procedure TWriteForm.WriteItem(OPCServer: TOPCDAServer; OPCGroup: TdOPCGroup;
  ItemId: string);
begin
  fGroupDelete := false;
  fOPCServer := OPCServer;
  fOPCGroup := OPCGroup;
  if fOPCGroup = nil then // if we don't have a group name
  begin // make a group
    fGroupDelete := true;
    fOPCGroup := fOPCServer.OPCGroups.Add('W-XXX-1064',false,false);  // hope, no such name exists
    fOPCGroup.OPCItems.AddItem(ItemId);
    fOPCGroup.IsActive := false;
  end;
  fOPCItem := fOPCGroup.OPCItems.FindOPCItem(ItemId);
  if fOPCItem <> nil then
    ReadCurrentValue
  else
  begin
    ShowMessage('Item not found in group');
    BackToParentForm;
  end;
end;

procedure TWriteForm.ReadCurrentValue;
var
  IL: TdOPCItemList;
  VType : integer;
begin
  LItemId.Text := fOPCItem.ItemName +' ('+ fOPCItem.ItemId +')';
  IL := TdOPCItemList.create(fOPCItem);
  fOPCGroup.SyncRead(IL, true);
  IL.CheckOPCErrors('Read item value', false);
  VType := fOPCItem.BaseDatatype;
  IL.Free;
  if fOPCServer.Protocol = coUA then
    eDatatType.Text := uaDataTypeToStr(VType.ToString)
  else
    eDatatType.Text := VarTypeAsText(VType);
  eCurrentValue.Text := fOPCItem.ValueStr;
  eNewValue.Text := eCurrentValue.Text;
  fOPCItem.WantValue := fOPCItem.Value;
end;

function VarFromStrings(aVarType: TVarType; List: TStrings): OleVariant;
var
  Count, I: integer;
begin
  List.Text := trim(List.Text); // delete all empty strings
  Count := List.Count;
  if Count = 0 then
  begin
    result := NULL;
    Exit;
  end;
  result := VarArrayCreate([0, Count - 1], aVarType and VarTypeMask);
  for I := 0 to Count - 1 do
    result[I] := List[I];
end;

function IsArray(Vt: TVarType): boolean;
begin
  result := (Vt and varArray) = varArray;
end;

procedure TWriteForm.EditButton1Click(Sender: TObject);
var
  StrList: TStringList;
  VType : integer;
begin
  VType := VarType(fOPCItem.WantValue);
  if IsArray(VType) then
  begin
    StrList := TStringList.create;
    StrList.StrictDelimiter := true;
    StrList.Delimiter := ';';
    StrList.DelimitedText := eNewValue.Text;
    fOPCItem.WantValue := VarFromStrings(VType, StrList);
    StrList.DisposeOf;
  end
  else
  begin
    if VType = varNull then
      VType := varOleStr;
    fOPCItem.WantValue := VarAsType(eNewValue.Text, VType);
  end;
  fOPCItem.WriteSync(fOPCItem.WantValue);
  eCurrentValue.Text := VarToAStr(fOPCItem.WantValue);
end;

end.
